@@ -139,7 +139,6 @@ static const char *acquire_error_str(int error)
 		return "lease io error";
 
 	case SANLK_LEADER_DIFF:
-	case SANLK_LEADER_MAGIC:
 	case SANLK_LEADER_VERSION:
 	case SANLK_LEADER_SECTORSIZE:
 	case SANLK_LEADER_LOCKSPACE:
@@ -148,6 +147,9 @@ static const char *acquire_error_str(int error)
 	case SANLK_LEADER_CHECKSUM:
 		return "lease data invalid";
 
+	case SANLK_LEADER_MAGIC:
+		return "lease not found";
+
 	default:
 		return "";
 	};
@@ -1805,6 +1807,7 @@ static void cmd_write_resource(struct task *task, struct cmd_args *ca)
 	struct sanlk_resource res;
 	int token_len, disks_len;
 	int num_hosts, max_hosts;
+	int write_clear = 0;
 	int j, fd, rv, result;
 
 	fd = client[ca->ci_in].fd;
@@ -1868,6 +1871,9 @@ static void cmd_write_resource(struct task *task, struct cmd_args *ca)
 	num_hosts = ca->header.data;
 	max_hosts = ca->header.data2;
 
+	if (ca->header.cmd_flags & SANLK_WRITE_CLEAR)
+		write_clear = 1;
+
 	rv = open_disks(token->disks, token->r.num_disks);
 	if (rv < 0) {
 		result = rv;
@@ -1876,7 +1882,7 @@ static void cmd_write_resource(struct task *task, struct cmd_args *ca)
 
 	token->io_timeout = DEFAULT_IO_TIMEOUT;
 
-	result = paxos_lease_init(task, token, num_hosts, max_hosts);
+	result = paxos_lease_init(task, token, num_hosts, max_hosts, write_clear);
 
 	close_disks(token->disks, token->r.num_disks);
  reply:
@@ -2079,6 +2085,7 @@ static int print_state_daemon(char *str)
 		 "mlock_level=%d "
 		 "quiet_fail=%d "
 		 "debug_renew=%d "
+		 "renewal_history_size=%d "
 		 "gid=%d "
 		 "uid=%d "
 		 "sh_retries=%d "
@@ -2099,6 +2106,7 @@ static int print_state_daemon(char *str)
 		 com.mlock_level,
 		 com.quiet_fail,
 		 com.debug_renew,
+		 com.renewal_history_size,
 		 com.gid,
 		 com.uid,
 		 com.sh_retries,
@@ -2240,6 +2248,25 @@ static int print_state_host(struct host_status *hs, char *str)
 	return strlen(str) + 1;
 }
 
+static int print_state_renewal(struct renewal_history *hi, char *str)
+{
+	memset(str, 0, SANLK_STATE_MAXSTR);
+
+	snprintf(str, SANLK_STATE_MAXSTR-1,
+		 "timestamp=%llu "
+		 "read_ms=%d "
+		 "write_ms=%d "
+		 "next_timeouts=%d "
+		 "next_errors=%d",
+		 (unsigned long long)hi->timestamp,
+		 hi->read_ms,
+		 hi->write_ms,
+		 hi->next_timeouts,
+		 hi->next_errors);
+
+	return strlen(str) + 1;
+}
+
 static void send_state_daemon(int fd)
 {
 	struct sanlk_state st;
@@ -2364,6 +2391,26 @@ static void send_state_host(int fd, struct host_status *hs, int host_id)
 		send(fd, str, str_len, MSG_NOSIGNAL);
 }
 
+static void send_state_renewal(int fd, struct renewal_history *hi)
+{
+	struct sanlk_state st;
+	char str[SANLK_STATE_MAXSTR];
+	int str_len;
+
+	memset(&st, 0, sizeof(st));
+
+	st.type = SANLK_STATE_RENEWAL;
+	st.data64 = hi->timestamp;
+
+	str_len = print_state_renewal(hi, str);
+
+	st.str_len = str_len;
+
+	send(fd, &st, sizeof(st), MSG_NOSIGNAL);
+	if (str_len)
+		send(fd, str, str_len, MSG_NOSIGNAL);
+}
+
 static void cmd_status(int fd, struct sm_header *h_recv, int client_maxi)
 {
 	struct sm_header h;
@@ -2475,6 +2522,97 @@ static void cmd_host_status(int fd, struct sm_header *h_recv)
 		free(status);
 }
 
+static void cmd_renewal(int fd, struct sm_header *h_recv)
+{
+	struct sm_header h;
+	struct sanlk_lockspace lockspace;
+	struct space *sp;
+	uint32_t io_timeout = 0;
+	struct renewal_history *history = NULL;
+	struct renewal_history *hi;
+	int history_size, history_prev, history_next;
+	int i, rv, len;
+
+	memset(&h, 0, sizeof(h));
+	memcpy(&h, h_recv, sizeof(struct sm_header));
+	h.version = SM_PROTO;
+	h.length = sizeof(h);
+	h.data = 0;
+
+	if (!com.renewal_history_size)
+		goto fail;
+
+	len = sizeof(struct renewal_history) * com.renewal_history_size;
+
+	history = malloc(len);
+	if (!history) {
+		h.data = -ENOMEM;
+		goto fail;
+	}
+
+	rv = recv(fd, &lockspace, sizeof(struct sanlk_lockspace), MSG_WAITALL);
+	if (rv != sizeof(struct sanlk_lockspace)) {
+		h.data = -ENOTCONN;
+		goto fail;
+	}
+
+	pthread_mutex_lock(&spaces_mutex);
+	sp = find_lockspace(lockspace.name);
+	if (sp) {
+		history_size = sp->renewal_history_size;
+		history_prev = sp->renewal_history_prev;
+		history_next = sp->renewal_history_next;
+		io_timeout = sp->io_timeout;
+
+		if (history_size != com.renewal_history_size) {
+			log_error("mismatch history size");
+			history_size = 0;
+			history_prev = 0;
+			history_next = 0;
+		} else {
+			memcpy(history, sp->renewal_history, len);
+		}
+	}
+	pthread_mutex_unlock(&spaces_mutex);
+
+	if (!sp) {
+		h.data = -ENOSPC;
+		goto fail;
+	}
+
+	if (!history_size || (!history_prev && !history_next))
+		goto fail;
+
+	h.data2 = io_timeout;
+
+	send(fd, &h, sizeof(h), MSG_NOSIGNAL);
+
+	/* If next slot is non-zero, then we've wrapped and
+	   should begin sending history from next to end
+	   before sending from 0 to prev. */
+
+	if (history[history_next].timestamp) {
+		for (i = history_next; i < history_size; i++) {
+			hi = &history[i];
+			send_state_renewal(fd, hi);
+		}
+	
+	}
+	for (i = 0; i < history_next; i++) {
+		hi = &history[i];
+		send_state_renewal(fd, hi);
+	}
+
+	if (history)
+		free(history);
+	return;
+ fail:
+	send(fd, &h, sizeof(h), MSG_NOSIGNAL);
+
+	if (history)
+		free(history);
+}
+
 static char send_data_buf[LOG_DUMP_SIZE];
 
 static void cmd_log_dump(int fd, struct sm_header *h_recv)
@@ -2715,6 +2853,10 @@ void call_cmd_daemon(int ci, struct sm_header *h_recv, int client_maxi)
 		strcpy(client[ci].owner_name, "host_status");
 		cmd_host_status(fd, h_recv);
 		break;
+	case SM_CMD_RENEWAL:
+		strcpy(client[ci].owner_name, "renewal");
+		cmd_renewal(fd, h_recv);
+		break;
 	case SM_CMD_LOG_DUMP:
 		strcpy(client[ci].owner_name, "log_dump");
 		cmd_log_dump(fd, h_recv);
